from pathlib import Path
from sys import path as syspath
from os.path import join
import time
import pygame


class Game:

    def __init__(self, width, height, importName, maxFPS=60, title="Game", path=str(Path(__file__).parent)):
        self.__width = width
        self.__height = height

        self.maxFPS = maxFPS

        syspath.append(path)

        self.__mainLib = __import__(importName)
        
        img = pygame.image.load(join(path, "templates/infinity.png"))
        pygame.display.set_icon(img)

        pygame.init()
        
        self.info = pygame.display.Info()
        self.screen = pygame.display.set_mode((width, height))
        pygame.display.set_caption(title)
        
        self.isRunning = False
        

    def run(self):
        self.__mainLib.setup()
        self.isRunning = True
        while self.isRunning:
            self.runUpdateCycle()
            
        pygame.quit()
            

    def runUpdateCycle(self):
        currentFrame = 0
        while currentFrame <= self.maxFPS:
            start = time.time()
            currentFrame = currentFrame + 1
            keys_up = []
            keys_down = []
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.isRunning = False
                if event.type == pygame.KEYUP:
                    keys_up.append(chr(event.key))
                elif event.type == pygame.KEYDOWN:
                    keys_down.append(chr(event.key))
            self.__mainLib.update(keys_up=keys_up, keys_down=keys_down)
            time.sleep(max(1./self.maxFPS - (time.time() - start), 0))

    def setGeometrics(self, width, height):
        self.__width, self.__height = (width, height)
        pygame.transform.scale(self.screen, (width, height))

    def getGeometrics(self):
        return (self.__width, self.__heigt)

    def setMouseIsVisible(self, visible):
        pygame.mouse.set_visible(visible)

    def setIcon(self, path, file=__file__):
        img = pygame.image.load(join(str(Path(file).parent), path))
        pygame.display.set_icon(img)

    def render(self):
        pygame.display.flip()

    class Drawer:

        def __init__(self, game):

            self.__game = game

        def Pixel(self, x, y, color):

            pygame.draw.rect(self.__game.screen, color.getColorCodeByDec(), pygame.Rect(x, y, 1, 1))

        def Rect(self, x1, y1, x2, y2, color):

            pygame.draw.rect(self.__game.screen, color.getColorCodeByDec(), pygame.Rect(x1, y1, x2 - x1, y2 - y1))

        def fill(self, color):

            self.__game.screen.fill(color.getColorCodeByDec())

    class Color:

        def __init__(self, r, g, b):

            self.__colorCode = [r, g, b]

        def getColorCodeByDec(self):

            return self.__colorCode

        def getColorCodeByHex(self):

            r, g, b = self.__colorCode

            r = str(hex(r)).replace("0x", "#")
            g = str(hex(g)).replace("0x", "")
            b = str(hex(b)).replace("0x", "")

            return r + g + b
